/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.http.HttpService;
import ru.ulstu.extractor.http.JsonTimeSeries;
import ru.ulstu.extractor.model.TimeSeries;
import ru.ulstu.extractor.model.TimeSeriesValue;
import ru.ulstu.extractor.repository.TimeSeriesRepository;
import ru.ulstu.extractor.repository.TimeSeriesValueRepository;
import ru.ulstu.extractor.ts.TimeSeriesDateMapper;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class TimeSeriesService {
    private final static Logger LOG = LoggerFactory.getLogger(TimeSeriesService.class);
    private final TimeSeriesRepository timeSeriesRepository;
    private final TimeSeriesValueRepository timeSeriesValueRepository;
    private final TimeSeriesDateMapper.TimeSeriesInterval timeSeriesInterval = TimeSeriesDateMapper.TimeSeriesInterval.HOUR;
    private final HttpService httpService;
    private final static String TIME_SERIES_SERVICE_URL = "http://time-series.athene.tech/api/1.0/add-time-series?setKey=git-extractor";

    public TimeSeriesService(TimeSeriesRepository timeSeriesRepository,
                             TimeSeriesValueRepository timeSeriesValueRepository,
                             HttpService httpService) {
        this.timeSeriesRepository = timeSeriesRepository;
        this.timeSeriesValueRepository = timeSeriesValueRepository;
        this.httpService = httpService;
    }

    /**
     * Сохранить список временных рядов
     *
     * @param timeSeriesValues
     * @return
     */
    @Transactional
    public List<TimeSeries> save(Map<String, List<TimeSeriesValue>> timeSeriesValues) {
        List<TimeSeries> results = new ArrayList<>();
        for (Map.Entry<String, List<TimeSeriesValue>> entry : timeSeriesValues.entrySet()) {
            results.add(save(entry.getKey(), entry.getValue()));
        }
        return results;
    }

    @Transactional
    public TimeSeries save(String timeSeriesName, List<TimeSeriesValue> timeSeriesValues) {
        LOG.debug("Start save {} time series with {} time series values ", timeSeriesName, timeSeriesValues.size());
        final TimeSeries timeSeries = findOrCreate(timeSeriesName);
        List<TimeSeriesValue> timeSeriesValuesToRemove = timeSeries.getValues();
        timeSeries.setValues(timeSeriesValues);
        LOG.debug("Save time series {} ", timeSeries.getName());
        TimeSeries savedTimeSeries = timeSeriesRepository.save(timeSeries);
        LOG.debug("Clear {} time series values ", timeSeriesValuesToRemove.size());
        timeSeriesValueRepository.deleteAll(timeSeriesValuesToRemove);
        sendToTimeSeriesService(savedTimeSeries);
        return savedTimeSeries;
    }

    public TimeSeries findOrCreate(String timeSeriesName) {
        Optional<TimeSeries> maybeTimeSeries = timeSeriesRepository.findByName(timeSeriesName);
        if (maybeTimeSeries.isPresent()) {
            LOG.debug("TimeSeries {} exists.", maybeTimeSeries.get().getName());
            return maybeTimeSeries.get();
        }
        return timeSeriesRepository.save(new TimeSeries(timeSeriesName));
    }

    public List<TimeSeriesValue> save(List<TimeSeriesValue> timeSeriesValues) {
        return timeSeriesValues.stream()
                .map(timeSeriesValue -> {
                    timeSeriesValue.setValue((timeSeriesValue.getValue()));
                    timeSeriesValue.setDate((timeSeriesValue.getDate()));
                    return timeSeriesValueRepository.save(timeSeriesValue);
                }).collect(Collectors.toList());
    }

    public void addTimeSeriesValue(String timeSeriesName, Date date, Double value) {
        LOG.debug("Start add time series values to {} time series values ", timeSeriesName);
        TimeSeries timeSeries = findOrCreate(timeSeriesName);
        timeSeriesValueRepository.save(new TimeSeriesValue(timeSeries, date, value));
    }

    public List<TimeSeriesValue> findAll() {
        return timeSeriesValueRepository.findAll();
    }

    public TimeSeriesDateMapper.TimeSeriesInterval getTimeSeriesInterval() {
        return timeSeriesInterval;
    }

    private void sendToTimeSeriesService(TimeSeries timeSeries) {
        new Thread(() -> {
            try {
                httpService.post(TIME_SERIES_SERVICE_URL, new JSONObject(new JsonTimeSeries(timeSeries)));
                LOG.debug("Успешно отправлен на сервис");
            } catch (Exception ex) {
                LOG.debug(ex.getMessage());
            }
        }).start();
    }
}
